      subroutine nnakk(ip,mu,ub,nb,pb,u,n,p,e,ne,q,e0,mass,wdth)
c----------------------------------------------------------------------c
c Returns a four vector array e=ub*Gamma^n*u*(-g_mn)/q^2,
c and its four momentum q with 
c    Gamma^n=e0*[F_1(q^2)*gamma^n+F_2(q^2)*i*sigma^nm*q_m]
c representing an effective nucleon-nucleon-photon coupling.
c Factor 1/(2m) included in F_2. Based on a routine btwan.f.
c Input: 
c        ip = 1/-1 if bar spinor u represents a particle/antiparticle,
c        mu - mass of a nucleon reprezented by u and ub,
c        ub,u - nucleon spinors,
c        pb,p - four momenta,
c        nb,ne,n - numbers of polarization states,
c        e0 - electric charge,
c        mass,wdth - not used,
c        ft1,ft2,fs1,fs2 - functions for computing nucleon form factors
c                          in the t- and s-channel.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 19 Feb./13 March, 2014
c----------------------------------------------------------------------c

      implicit none

      integer :: ip,nb,ne,n

      integer, parameter :: dp=kind(1.d0)

      real(dp) :: e0,mu,mass,wdth
      complex(dp) :: f1,f2
      real(dp), dimension(0:3) :: pb,p,q,pe
      complex(dp), dimension(4,nb) :: ub
      complex(dp), dimension(4,n) :: u
      complex(dp), dimension(0:3,ne) :: e

c Local variables:

      integer ib,ie,i,l
      real(dp) :: den
      complex(dp), parameter :: icom=(0.d0,1.d0)
      complex(dp) :: f1l,f1r,f2l,f2r
      complex(dp), dimension(0:3,2,2) :: a,d

c      q=pb+p
      q=ip*(pb+p)

      den=q(0)**2-q(1)**2-q(2)**2-q(3)**2

      call nuclff_phok(mu,den,f1,f2)

      f1r=e0*f1
      f1l=f1r
      f2r=e0*f2
      f2l=f2r

      if (ip == -1) then
       f1r=conjg(f1r)
       f1l=conjg(f1l)
       f2r=conjg(f2r)
       f2l=conjg(f2l)
      end if

      pe=-q ! Vertex is parametrized in terms of outgoing photon four momentum

      a(0,1,:)=(/cmplx(pe(3),0.d0,dp),cmplx(pe(1),-pe(2),dp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),dp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.d0,pe(2),dp),cmplx(pe(0)-pe(3),0.d0,dp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.d0,dp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.d0,-pe(1),dp),cmplx(0.d0,-pe(0)+pe(3),dp)/)
      a(2,2,:)=(/cmplx(0.d0,pe(0)+pe(3),dp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.d0,dp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      ie=0

      do i=1,n
       do ib=1,nb
        ie=ie+1
        do l=0,3
         e(l,ie)=f2l*(ub(1,ib)*(a(l,1,1)*u(1,i)+a(l,1,2)*u(2,i))
     &               +ub(2,ib)*(a(l,2,1)*u(1,i)+a(l,2,2)*u(2,i)))
     &          +f2r*(ub(3,ib)*(d(l,1,1)*u(3,i)+d(l,1,2)*u(4,i))
     &               +ub(4,ib)*(d(l,2,1)*u(3,i)+d(l,2,2)*u(4,i)))
        end do
        e(0,ie)=e(0,ie)+f1l*(ub(3,ib)*u(1,i)+ub(4,ib)*u(2,i))
     &                 +f1r*(ub(1,ib)*u(3,i)+ub(2,ib)*u(4,i))
        e(1,ie)=e(1,ie)-f1l*(ub(3,ib)*u(2,i)+ub(4,ib)*u(1,i))
     &                 +f1r*(ub(1,ib)*u(4,i)+ub(2,ib)*u(3,i))
        e(2,ie)=e(2,ie)-f1l*(-ub(3,ib)*u(2,i)+ub(4,ib)*u(1,i))*icom
     &                 +f1r*(-ub(1,ib)*u(4,i)+ub(2,ib)*u(3,i))*icom
        e(3,ie)=e(3,ie)-f1l*(ub(3,ib)*u(1,i)-ub(4,ib)*u(2,i))
     &                 +f1r*(ub(1,ib)*u(3,i)-ub(2,ib)*u(4,i))
        e(:,ie)=-e(:,ie)/den
       end do
      end do

      end
