      program carlomat
c----------------------------------------------------------------------c
c Automized calculation of the cross section of SM process
c
c                 f_1 f_2 -> f_3 f_4 ... f_next
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 9 February, 2006; modified 
c----------------------------------------------------------------------c

      use mytypes
      use extpart
      use schkins
      use tchkins
      use gchkins

      implicit none

      character(80) :: process

c Specify a process below. The maximum number of external particles 
c is 12. 
c Standard Model particles include (notation adopted from MadGraph)
c     Quarks:   d u s c b t d~ u~ s~ c~ b~ t~
c     Leptons:  e- mu- ta- e+ mu+ ta+ ve vm vt ve~ vm~ vt~
c     Bosons:   g a z w+ w- h
c Other particles:
c     Nucleons: p, p~, n, n~
c     Mesons: pi-, pi+, pi0, k-, k+, k0, k0~, rho-, rho+, rho0, ome, phi,
c             rho1, rho2
c
c      process='e+ e- -> e+ e- pi+ pi- a'
c      process='e+ e- -> e+ e- mu+ mu-'
c      process='e+ e- -> e+ e- mu+ mu- a'
c      process='e+ e- -> e+ e- e+ e- mu+ mu-'
      process='e+ e- -> mu+ mu- ta+ ta- a'
c      process='e+ e- -> mu+ mu- ta+ ta- u~ u a'
c      process='e+ e- -> e+ e- mu+ mu- ta+ ta- pi+ pi- a'
c      process='e+ e- -> b e+ ve b~ mu- vm~' 
c      process='e+ e- -> b u d~ b~ mu- vm~ a' 
c      process='e+ e- -> b u d~ b~ mu- vm~ ta+ ta- pi+ pi- a' 
c      process='e+ e- -> b u d~ b~ mu- vm~ ta+ ta- a' 
c      process='e+ e- -> b e+ ve b~ e- ve~' 

c Take into account photon radiation from the initial state only (isr=1)/
c final state only (isr=2)/ both (isr=else)? Relevant only for processes with
c external photons. Makes sens only if ISR and FSR form gauge independent
c subsets of the Feynman diagrams

      isr=0

c Generate s-channel kinemtics, possible for any proces, but compilation and
c computation of the generated MC kinematical routines might be very time 
c consuming, ischnl=1 (yes, kinematics based on topologies of the Feynman
c diagrams)/2 (yes, kinematics taking into account poles due to Feynman 
c propagators of unstable heavy particles which decay into on shell final 
c state particles)/else (no)?
      
      ischnl=1

c Generate single photon radiation kinematics, for processes with one photon 
c or gluon in the final state, igchnl=1 (yes)/else (no)?

      igchnl=1

c Generate t-channel kinematics, for processes with identical fermion-antifermion
c pairs in the initial and final state, itchnl=1 (yes)/else (no)?

      itchnl=0

c Use quartic precision for the particle four momenta, masses and widths,
c iqprec=1 (yes)/else (no)?:
 
      iqprec=0

c Short name of the process:

      prcsnm='auto'  ! If prcsnm='auto' then it is made automatically.
 
c Monte Carlo summing over polarizations: imcs=1 (yes)/else (no)?

      imcs=1

c Take into account the Cabibbo-Kobayashi-Maskawa mixing: ickm=1(yes)/else(no)?

      ickm=0

c Include the anomalous Wtb coupling: iwtb=1 (Yes)/else (No)? 
c (see K. Kolodziej, Phys. Lett. B584 (2004) 89)

      iwtb=0

      call readpart(process)

      if (ischnl == 2) then
       call genps(next-nisp)
      end if

      call genmat

      end

