      module inprms

      use mathprms

      real(dp), parameter ::  
     &     mz=91.1876d0,  gamz=2.4952d0, ! Z mass and width
     &     mw=80.379d0,   !gamw=2.12d0,  ! W mass and width
     &     ialph0=137.035999139d0, ! an inverse of alpha in the Thomson limit
     &     gmu=1.1663787d-5,        ! the Fermi coupling constant
     &     mh=125.18d0              ! Higgs mass

c Photon and gluon masses and widths

      real(dp), parameter :: ma=zero, gama=zero,
     &                       mg=zero, gamg=zero,
     &                       sqsc=1.d-8 ! minimum virtuality

c Quark masses that will be changed to corresponding cteq6f values  if ihad > 0: 
      real(dp), parameter :: 
     &       mu0=5.d-3, md0=10.d-3, ms0=200.d-3, 
     &       mc0=1.275d0, mb0=4.18d0


c Fermion masses:

      real(dp), parameter :: 
     &       me=0.5109989461d-3, mmu=105.6583745d-3, mtau=1.77686d0,
     &       mt=173.0d0
c     &       alphas=0.1176d0 ! at mz

      real(dp) :: mu, md, ms, mc, mb,      
     &            alphas=0.1181d0, ! at mz, the same as in cteq6f,
     &            x0pdf=1.d-6     ! min of x in cteq6f and mstw

c Nucleon masses:
c Note that nucleon magnetic moments and vector boson masses are specified
c in nuclff_phok.f in directory carloilb

      real(dp), parameter :: mp=938.272081d-3, mn=939.565413d-3

c Fermion widths:

      real(dp), parameter :: 
     &       game=zero       , gammu=zero       , gamtau=zero,
     &       gamu=zero       , gamc=zero        , 
     &       gamd=zero       , gams=zero        , gamb=zero

      real(dp) :: gamt=0.d0, gamh=0.d0, gamw=2.085d0, gmtb
c      real(dp) :: gamt=0.d0, gamh=0.d0, gamw=0.d0

c No of colours:

      real(dp), parameter :: ncol=3.d0

c Conversion costant (pb*GeV^2):

      real(dp), parameter ::   convc=0.3893793656d12

c EW mixing parameter:

      real(dp), parameter ::   sw2=1.d0-(mw/mz)**2,
     &   ialphw=pi/(sqrt2*sw2*gmu*mw**2), ! This is exactly the Gmu scheme
     &   ialpho=0.1d0                     ! Arbitrary choice

      real(dp) :: ralpw0

c Magnitudes of the CKM mixing parameters:

      real(dp), parameter ::   
     &          V_ud=0.97427d0,  V_us=0.22534d0,  V_ub=0.00351d0,
     &          V_cd=0.22520d0,  V_cs=0.97344d0,  V_cb=0.0412d0,
     &          V_td=0.00867d0,  V_ts=0.0404d0,   V_tb=0.999146d0

      integer :: igauge

c SM couplings and widths are not running, therefore

      integer, parameter :: iaww=0,izww=0,iggg=0,
     &                      ihzz=0,ihww=0,ihhh=0,
     &                      iaaww=0,iazww=0,izzww=0,iwwww=0,igggg=0,
     &                      ihhww=0,ihhzz=0,ihhhh=0,
     &                      igamt=0,igamz=0,igamw=0,igamh=0

c HLS couplings:

      real(dp), parameter :: f_pi=92.42d-3,
     & c1mc2=1.226d0,c3=0.920d0,c4=c3,
     & g_HLS=5.578d0,
     & Gcap=-g_HLS/(4.d0*pi**2*f_pi)*(c3+c4)/2.d0,
     & Ccap=-ncol*g_HLS**2/(4.d0*pi**2*f_pi)*c3,
     & g_rhopipi=6.505d0,
     & g_omepipi=0.408d0,
     & g_phipipi=7.24d-3,
     & g_phikk=4.471d0,
     & g_piaa=-ncol/(24.d0*pi**2*f_pi)*(1.d0-c4),
     & g_rhoapi=Gcap/2.d0,
     & g_omeapi=3.d0*Gcap/2.d0,
     & g_omerhopi=Ccap/2.d0,
     & g_apipipi=-ncol/(12.d0*pi**2*f_pi**3)*(1.d0-0.75d0*(c1mc2+c4)),
     & g_omepipipi=-3.d0*ncol*g_HLS/(16.d0*pi**2*f_pi**3)*(c1mc2-c3),
     & f_arho=2.398d0*g_HLS*f_pi**2,
     & f_aome=f_arho/3.d0,
     & f_aphi=-7.09d-2

c Meson masses:

      real(dp), parameter :: mpi=139.57018d-3, mpi0=134.9766d-3,
     &                       mk=493.677d-3,mk0=497.614d-3,
     &                       mrho=775.26d-3,mome=782.65d-3,
     &                       mphi=1019.461d-3,
     &                       mrh1=1465.d-3,mrh2=1720.d-3
c     &                       mrho=774.37d-3,mome=782.65d-3,

c Meson widths:

      real(dp), parameter :: gampi=zero,gmrh=149.1d-3,
     &                       gmom=8.49d-3,
     &                       gmph=4.266d-3,
     &                       gmr1=400.d-3,gmr2=250.d-3
c      real(dp), parameter :: gampi=zero,gmrh=144.21d-3,gmom=8.49d-3

c     &       gampi=1.054571628d-25/2.6033d-8

      integer :: igmrh,igmom,igmph,igmr1,igmr2,
     &           imrho,imome,imphi,imrh1,imrh2
      real(dp) :: wmrho,dmrho,wmome,dmome,wmphi,dmphi,
     &            wmrh1,dmrh1,wmrh2,dmrh2
      integer, parameter :: izero=0,ima=0      ! photon included with zero width,
      real(dp), parameter :: wma=0.d0,dma=0.d0 ! weight=1 and no phase factor

c Masses for the quadruple precision kinematics, should be the same as those
c defined above. Note that quark masses, other than mt, are automatically 
c changed in the matrix element computation, if ihad > 0 (see parfixkk.f):

      real(qp), parameter :: zero_qp=0.q0, ma_qp=0.q0, mg_qp=0.q0
      real(qp) :: mz_qp, mw_qp, mh_qp,
     & me_qp, mmu_qp, mtau_qp,
     & mu_qp, md_qp, ms_qp, mc_qp, mb_qp, mt_qp,
     & mp_qp, mn_qp,
     & mpi_qp, mpi0_qp, mk_qp, mk0_qp,
     & mrho_qp, mome_qp, mphi_qp, mrh1_qp, mrh2_qp

      end module inprms

